import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _extends from "@babel/runtime/helpers/extends";
import _defineProperty from "@babel/runtime/helpers/defineProperty";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

import React, { useReducer } from 'react';
import PropTypes from 'prop-types';
import useFormApi from "@data-driven-forms/react-form-renderer/use-form-api";
import FieldArray from "@data-driven-forms/react-form-renderer/field-array";
import Grid from "@material-ui/core/esm/Grid";
import Button from "@material-ui/core/esm/Button";
import Typography from "@material-ui/core/esm/Typography";
import FormControl from "@material-ui/core/esm/FormControl";
import FormHelperText from "@material-ui/core/esm/FormHelperText";
import IconButton from "@material-ui/core/esm/IconButton";
import { makeStyles } from '@material-ui/core/styles';
import RedoIcon from '@material-ui/icons/Redo';
import UndoIcon from '@material-ui/icons/Undo';
import useFieldApi from "@data-driven-forms/react-form-renderer/use-field-api";
import FormFieldGrid from '../form-field-grid/form-field-grid';
import clsx from 'clsx';
var useFielArrayStyles = makeStyles({
  formControl: {
    width: '100%'
  },
  centerText: {
    display: 'flex',
    justifyContent: 'center'
  },
  buttonsToEnd: {
    display: 'flex',
    justifyContent: 'flex-end'
  },
  header: {
    display: 'flex'
  },
  label: {
    flexGrow: 1
  },
  fieldArrayGroup: {
    marginBottom: 32
  }
});

var ArrayItem = function ArrayItem(_ref) {
  var fields = _ref.fields,
      fieldIndex = _ref.fieldIndex,
      name = _ref.name,
      remove = _ref.remove,
      length = _ref.length,
      minItems = _ref.minItems,
      removeLabel = _ref.removeLabel,
      FieldContainerProps = _ref.FieldContainerProps,
      FieldGroupGridProps = _ref.FieldGroupGridProps,
      RemoveButtonGridProps = _ref.RemoveButtonGridProps,
      RemoveButtonProps = _ref.RemoveButtonProps;

  var _useFormApi = useFormApi(),
      renderForm = _useFormApi.renderForm;

  var classes = useFielArrayStyles();
  var editedFields = fields.map(function (field, index) {
    var computedName = field.name ? "".concat(name, ".").concat(field.name) : name;
    return _objectSpread(_objectSpread({}, field), {}, {
      name: computedName,
      key: "".concat(computedName, "-").concat(index)
    });
  });
  return /*#__PURE__*/React.createElement(Grid, _extends({
    container: true,
    spacing: 3
  }, FieldContainerProps), /*#__PURE__*/React.createElement(Grid, _extends({
    item: true,
    xs: 12
  }, FieldGroupGridProps), /*#__PURE__*/React.createElement(Grid, {
    container: true,
    spacing: 3
  }, renderForm([editedFields]))), /*#__PURE__*/React.createElement(Grid, _extends({
    item: true,
    xs: 12,
    className: classes.buttonsToEnd
  }, RemoveButtonGridProps), /*#__PURE__*/React.createElement(Button, _extends({
    color: "secondary",
    onClick: function onClick() {
      return remove(fieldIndex);
    },
    disabled: length <= minItems
  }, RemoveButtonProps), removeLabel)));
};

ArrayItem.propTypes = {
  name: PropTypes.string,
  fieldIndex: PropTypes.number.isRequired,
  fields: PropTypes.arrayOf(PropTypes.object),
  remove: PropTypes.func.isRequired,
  length: PropTypes.number,
  minItems: PropTypes.number,
  removeLabel: PropTypes.node.isRequired,
  FieldContainerProps: PropTypes.object,
  FieldGroupGridProps: PropTypes.object,
  RemoveButtonGridProps: PropTypes.object,
  RemoveButtonProps: PropTypes.object
};
ArrayItem.defaultProps = {
  FieldContainerProps: {},
  FieldGroupGridProps: {},
  RemoveButtonGridProps: {},
  RemoveButtonProps: {}
};
var defaultButtonLabels = {
  add: 'ADD',
  remove: 'REMOVE'
};
var initialState = {
  index: 0,
  history: []
};
export var reducer = function reducer(state, _ref2) {
  var type = _ref2.type,
      action = _ref2.action;

  switch (type) {
    case 'redo':
      return _objectSpread(_objectSpread({}, state), {}, {
        index: state.index + 1
      });

    case 'action':
      return {
        index: state.index + 1,
        history: [].concat(_toConsumableArray(state.history.slice(0, state.index)), [action])
      };

    case 'undo':
      return _objectSpread(_objectSpread({}, state), {}, {
        index: state.index - 1
      });

    case 'resetHistory':
      return _objectSpread(_objectSpread({}, state), {}, {
        history: state.history.slice(0, state.index)
      });

    default:
      return state;
  }
};

var DynamicArray = function DynamicArray(_ref3) {
  var props = _extends({}, _ref3);

  var _useFieldApi = useFieldApi(props),
      arrayValidator = _useFieldApi.arrayValidator,
      label = _useFieldApi.label,
      description = _useFieldApi.description,
      formFields = _useFieldApi.fields,
      defaultItem = _useFieldApi.defaultItem,
      meta = _useFieldApi.meta,
      minItems = _useFieldApi.minItems,
      maxItems = _useFieldApi.maxItems,
      noItemsMessage = _useFieldApi.noItemsMessage,
      FormFieldGridProps = _useFieldApi.FormFieldGridProps,
      FormControlProps = _useFieldApi.FormControlProps,
      buttonLabels = _useFieldApi.buttonLabels,
      GridContainerProps = _useFieldApi.GridContainerProps,
      HeaderGridProps = _useFieldApi.HeaderGridProps,
      HeaderProps = _useFieldApi.HeaderProps,
      UndoButtonProps = _useFieldApi.UndoButtonProps,
      RedoButtonProps = _useFieldApi.RedoButtonProps,
      AddButtonProps = _useFieldApi.AddButtonProps,
      DescriptionGridProps = _useFieldApi.DescriptionGridProps,
      DescriptionProps = _useFieldApi.DescriptionProps,
      BodyGridProps = _useFieldApi.BodyGridProps,
      NoItemsProps = _useFieldApi.NoItemsProps,
      FormHelperTextGridProps = _useFieldApi.FormHelperTextGridProps,
      FormHelperTextProps = _useFieldApi.FormHelperTextProps,
      FieldContainerProps = _useFieldApi.FieldContainerProps,
      FieldGroupGridProps = _useFieldApi.FieldGroupGridProps,
      RemoveButtonGridProps = _useFieldApi.RemoveButtonGridProps,
      RemoveButtonProps = _useFieldApi.RemoveButtonProps,
      rest = _objectWithoutProperties(_useFieldApi, ["arrayValidator", "label", "description", "fields", "defaultItem", "meta", "minItems", "maxItems", "noItemsMessage", "FormFieldGridProps", "FormControlProps", "buttonLabels", "GridContainerProps", "HeaderGridProps", "HeaderProps", "UndoButtonProps", "RedoButtonProps", "AddButtonProps", "DescriptionGridProps", "DescriptionProps", "BodyGridProps", "NoItemsProps", "FormHelperTextGridProps", "FormHelperTextProps", "FieldContainerProps", "FieldGroupGridProps", "RemoveButtonGridProps", "RemoveButtonProps"]);

  var _useReducer = useReducer(reducer, initialState),
      _useReducer2 = _slicedToArray(_useReducer, 2),
      state = _useReducer2[0],
      dispatch = _useReducer2[1];

  var combinedButtonLabels = _objectSpread(_objectSpread({}, defaultButtonLabels), buttonLabels);

  var classes = useFielArrayStyles();
  var dirty = meta.dirty,
      submitFailed = meta.submitFailed,
      error = meta.error,
      submitError = meta.submitError;
  var isError = (dirty || submitFailed) && error && typeof error === 'string';
  return /*#__PURE__*/React.createElement(FormFieldGrid, _extends({}, FormFieldGridProps, {
    className: clsx(classes.fieldArrayGroup, FormFieldGridProps.classname)
  }), /*#__PURE__*/React.createElement(FormControl, _extends({
    component: "fieldset",
    error: isError || submitError
  }, FormControlProps, {
    className: clsx(classes.formControl, FormControlProps.className)
  }), /*#__PURE__*/React.createElement(FieldArray, {
    key: rest.input.name,
    name: rest.input.name,
    validate: arrayValidator
  }, function (_ref4) {
    var _ref4$fields = _ref4.fields,
        map = _ref4$fields.map,
        _ref4$fields$value = _ref4$fields.value,
        value = _ref4$fields$value === void 0 ? [] : _ref4$fields$value,
        push = _ref4$fields.push,
        remove = _ref4$fields.remove;

    var pushWrapper = function pushWrapper() {
      dispatch({
        type: 'resetHistory'
      });
      push(defaultItem);
    };

    var removeWrapper = function removeWrapper(index) {
      dispatch({
        type: 'action',
        action: {
          action: 'remove',
          value: value[index]
        }
      });
      remove(index);
    };

    var undo = function undo() {
      push(state.history[state.index - 1].value);
      dispatch({
        type: 'undo'
      });
    };

    var redo = function redo() {
      remove(value.length - 1);
      dispatch({
        type: 'redo'
      });
    };

    return /*#__PURE__*/React.createElement(Grid, _extends({
      container: true,
      spacing: 3
    }, GridContainerProps), /*#__PURE__*/React.createElement(Grid, _extends({
      item: true,
      xs: 12,
      className: classes.header
    }, HeaderGridProps), label && /*#__PURE__*/React.createElement(Typography, _extends({
      variant: "h6",
      className: classes.label
    }, HeaderProps), label), /*#__PURE__*/React.createElement(IconButton, _extends({
      color: "primary",
      "aria-label": "undo",
      component: "span",
      disabled: state.index === 0,
      onClick: undo
    }, UndoButtonProps), /*#__PURE__*/React.createElement(UndoIcon, null)), /*#__PURE__*/React.createElement(IconButton, _extends({
      color: "primary",
      "aria-label": "redo",
      component: "span",
      disabled: state.index === state.history.length,
      onClick: redo
    }, RedoButtonProps), /*#__PURE__*/React.createElement(RedoIcon, null)), /*#__PURE__*/React.createElement(Button, _extends({
      color: "primary",
      onClick: pushWrapper,
      disabled: value.length >= maxItems
    }, AddButtonProps), combinedButtonLabels.add)), description && /*#__PURE__*/React.createElement(Grid, _extends({
      item: true,
      xs: 12
    }, DescriptionGridProps), /*#__PURE__*/React.createElement(Typography, _extends({
      variant: "subtitle1"
    }, DescriptionProps), description)), /*#__PURE__*/React.createElement(Grid, _extends({
      item: true,
      xs: 12
    }, BodyGridProps), value.length <= 0 ? /*#__PURE__*/React.createElement(Typography, _extends({
      variant: "body1",
      gutterBottom: true,
      className: classes.centerText
    }, NoItemsProps), noItemsMessage) : map(function (name, index) {
      return /*#__PURE__*/React.createElement(ArrayItem, {
        key: name,
        fields: formFields,
        name: name,
        fieldIndex: index,
        remove: removeWrapper,
        length: value.length,
        minItems: minItems,
        removeLabel: combinedButtonLabels.remove,
        FieldContainerProps: FieldContainerProps,
        FieldGroupGridProps: FieldGroupGridProps,
        RemoveButtonGridProps: RemoveButtonGridProps,
        RemoveButtonProps: RemoveButtonProps
      });
    })), (isError || submitError) && /*#__PURE__*/React.createElement(Grid, _extends({
      item: true,
      xs: 12
    }, FormHelperTextGridProps), /*#__PURE__*/React.createElement(FormHelperText, FormHelperTextProps, error || submitError)));
  })));
};

DynamicArray.propTypes = {
  label: PropTypes.node,
  description: PropTypes.node,
  fields: PropTypes.arrayOf(PropTypes.object).isRequired,
  defaultItem: PropTypes.any,
  minItems: PropTypes.number,
  maxItems: PropTypes.number,
  noItemsMessage: PropTypes.node,
  FormControlProps: PropTypes.object,
  FormFieldGridProps: PropTypes.object,
  buttonLabels: PropTypes.object,
  GridContainerProps: PropTypes.object,
  HeaderGridProps: PropTypes.object,
  HeaderProps: PropTypes.object,
  UndoButtonProps: PropTypes.object,
  RedoButtonProps: PropTypes.object,
  AddButtonProps: PropTypes.object,
  DescriptionGridProps: PropTypes.object,
  DescriptionProps: PropTypes.object,
  BodyGridProps: PropTypes.object,
  NoItemsProps: PropTypes.object,
  FormHelperTextGridProps: PropTypes.object,
  FormHelperTextProps: PropTypes.object,
  FieldContainerProps: PropTypes.object,
  FieldGroupGridProps: PropTypes.object,
  RemoveButtonGridProps: PropTypes.object,
  RemoveButtonProps: PropTypes.object
};
DynamicArray.defaultProps = {
  maxItems: Infinity,
  minItems: 0,
  noItemsMessage: 'No items added',
  FormControlProps: {},
  FormFieldGridProps: {},
  GridContainerProps: {},
  HeaderGridProps: {},
  HeaderProps: {},
  UndoButtonProps: {},
  RedoButtonProps: {},
  AddButtonProps: {},
  DescriptionGridProps: {},
  DescriptionProps: {},
  BodyGridProps: {},
  NoItemsProps: {},
  FormHelperTextGridProps: {},
  FormHelperTextProps: {},
  FieldContainerProps: {},
  FieldGroupGridProps: {},
  RemoveButtonGridProps: {},
  RemoveButtonProps: {}
};
export default DynamicArray;